unit Extract4000;

interface
uses
  Int64Em, Compress, Struct, Extract, FileClass;

type
  PDecompressExtraData = ^TDecompressExtraData;
  TDecompressExtraData = record
    CurSlice, LastSlice: Integer;
    BytesLeft, Size, BytesWritten: Integer64;
  end;

  TFileExtractor4000 = class(TFileExtractor)
  protected
    DecompressData: TDecompressExtraData;
    Adler32: Longint;
    TestCompID: TCompID;
  public
    constructor Create(ADecompressorClass: TCustomDecompressorClass); override;
    destructor Destroy; override;
    procedure DecompressFile(const FL: TSetupFileLocationEntry; const DestFile: TFile;
      const ProgressProc: TExtractorProgressProc); override;
    procedure SeekTo(const FL: TSetupFileLocationEntry); override;
  end;

implementation
uses
  SysUtils, zlib, bzlib, Msgs, CmnFunc2, PathFunc, Main, MyTypes, Windows;

var
  SourceFile: string;
  SourceF: TFile; DestF:TFile;
  SourceFOpen:boolean;
  LastSourceDir: String;

function BZCustomInflateData(const ReadProc: TZReadProc; const WriteProc: TZWriteProc;
  const ExtraData: Longint; var Adler: Longint): Boolean; forward;

procedure SourceIsCorrupted;
begin
  raise Exception.Create(SSetupFileCorrupt);
end;

procedure ReadDiskSliceHeader;
var
  TestDiskSliceID: TDiskSliceID;
  DiskSliceHeader: TDiskSliceHeader;
begin
  if SourceF.CappedSize < SizeOf(TestDiskSliceID) + SizeOf(DiskSliceHeader) then
    SourceIsCorrupted;
  SourceF.ReadBuffer(TestDiskSliceID, SizeOf(TestDiskSliceID));
  if TestDiskSliceID <> DiskSliceID then
    SourceIsCorrupted;
  SourceF.ReadBuffer(DiskSliceHeader, SizeOf(DiskSliceHeader));
  if SourceF.CappedSize <> DiskSliceHeader.TotalSize then
    SourceIsCorrupted;
end;

function FindSlice(const ASlice: Integer): String;
var
  Major{, Minor}: Integer;
  {Prefix, Path,} F1, F2: String;
begin
{  if Ver>=4107 then Prefix := PathChangeExt(PathExtractName(SetupLdrOriginalFilename), '')
  else Prefix:=SetupHeader.BaseFilename;}
  Major := ASlice div SetupHeader.SlicesPerDisk + 1;
{  Minor := ASlice mod SetupHeader.SlicesPerDisk;
  if SetupHeader.SlicesPerDisk = 1 then
    F1 := Format('%s-%d.bin', [Prefix, Major])
  else
    F1 := Format('%s-%d%s.bin', [Prefix, Major,
      Chr(Ord('a') + Minor)]);}
  F1:=GetSliceName(ASlice);

  F2 := Format('..\DISK%d\', [Major]) + F1;
  Result := AddBackslash(LastSourceDir) + F1;
  if NewFileExists(Result) then Exit;
  Result := AddBackslash(SourceDir) + F1;
  if NewFileExists(Result) then Exit;
  Result := ExpandFilename(AddBackslash(LastSourceDir) + F2);
  if NewFileExists(Result) then Exit;
  Result := ExpandFilename(AddBackslash(SourceDir) + F2);
  if NewFileExists(Result) then Exit;
  if not InteractiveMode then raise Exception.Create('Could not find a necessary file: '+F1)
  else begin
    Result:=OpenFileDlg(F1,SourceDir);
    if Result='' then raise Exception.Create('Could not find a necessary file: '+F1);
    LastSourceDir:=PathExtractDir(Result);
  end;
//  Path := SourceDir;
{  if SelectDisk(Major, F1, Path) then begin
    LastSourceDir := Path;
    Result := AddBackslash(Path) + F1;
  end
  else
    Abort;}
end;

function zlibReadProc(var Buf; MaxBytes: Cardinal; ExtraData: Longint): Cardinal; far;
var
  Buffer: Pointer;
  Left, Res: Cardinal;
  NewSliceFilename: String;
begin
  Buffer := @Buf;
  with PDecompressExtraData(ExtraData)^ do begin
    Left := MaxBytes;
    if (BytesLeft.Hi = 0) and (BytesLeft.Lo < Left) then
      Left := BytesLeft.Lo;
    Result := Left;
    while Left <> 0 do begin
      Res:=SourceF.Read(Buffer^, Left);
      Dec64(BytesLeft, Res);
      if Left = Res then
        Break
      else begin
        Dec(Left, Res);
        Inc(Longint(Buffer), Res);
        { Go to next disk }
        if CurSlice >= LastSlice then
          { Already on the last slice, so the file must be corrupted... }
          SourceIsCorrupted
        else begin
//          CloseFile(SourceF);
          SourceFOpen := False;
          SourceF.Free;
          Inc(CurSlice);
          NewSliceFilename := FindSlice(CurSlice);
          if not QuietExtract then writeln('Reading slice '+NewSliceFilename);
//          AssignFile(SourceF, NewSliceFilename);
//          FileMode := fmOpenRead or fmShareDenyWrite;  Reset(SourceF, 1);
          SourceF:=TFile.Create(NewSliceFilename,fdOpenExisting,faRead,fsRead);
          SourceFOpen := True;
          ReadDiskSliceHeader;
        end;
      end;
    end;
  end;
(*  if zlibSetProgress then begin
    { Increment progress meter }
    IncProgress(Result);
    ProcessEvents;
  end;*)
end;

function zlibWriteProc(var Buf; BufSize: Cardinal; ExtraData: Longint): Cardinal; far;
begin
  with PDecompressExtraData(ExtraData)^ do begin
    Inc64(BytesWritten, BufSize);
    if Compare64(BytesWritten, Size) > 0 then
      SourceIsCorrupted;
    DestF.WriteBuffer(Buf, BufSize);
  end;
  Result := BufSize;
end;

///////////////////////       TFileExtractor4000        ///////////////////////////////

constructor TFileExtractor4000.Create(ADecompressorClass: TCustomDecompressorClass);
begin
end;

destructor TFileExtractor4000.Destroy;
begin
//  if SourceFOpen then
  // CloseFile(SourceF);
    FreeAndNil(SourceF);
end;

procedure TFileExtractor4000.SeekTo(const FL: TSetupFileLocationEntry);
begin
  if SetupLdrOffset1 = 0 then
    SourceFile := FindSlice(FL.FirstSlice)
  else
    SourceFile := SetupLdrOriginalFilename;

          { Open source file }
          SourceF:=TFile.Create(SourceFile,fdOpenExisting,faRead,fsRead);
//          AssignFile(SourceF, SourceFile);
//          FileMode := fmOpenRead or fmShareDenyWrite;  Reset(SourceF, 1);
          SourceFOpen := True;

  if SetupLdrOffset1 = 0 then
    ReadDiskSliceHeader;
  SourceF.Seek(SetupLdrOffset1 + FL.StartOffset);
  SourceF.ReadBuffer(TestCompID, SizeOf(TestCompID));
  if TestCompID <> ZLIBID then
    SourceIsCorrupted;
  with DecompressData do begin
    CurSlice := FL.FirstSlice;
    LastSlice := FL.LastSlice;
    BytesLeft := FL.ChunkCompressedSize;
    Size := FL.OriginalSize;
    BytesWritten.Hi := 0;
    BytesWritten.Lo := 0;
  end;
end;

procedure TFileExtractor4000.DecompressFile(const FL: TSetupFileLocationEntry; const DestFile: TFile;
  const ProgressProc: TExtractorProgressProc);
var
  CurFileDate, CurFileDate2: TFileTime;
begin
  DestF:=DestFile;
  try
    case SetupHeader.CompressMethod of
      cmZip: CustomInflateData(zlibReadProc, zlibWriteProc, Longint(@DecompressData), Adler32);
      cmBzip: BZCustomInflateData(zlibReadProc, zlibWriteProc, Longint(@DecompressData), Adler32);
    end;
  except
    on ECompressDataError do
      SourceIsCorrupted;
  end;
  if (Compare64(DecompressData.BytesWritten, FL.OriginalSize) <> 0) or
     (Adler32 <> FL.CRC) then
    SourceIsCorrupted;

  { Set time/date stamp }
  CurFileDate := FL.TimeStamp;
  LocalFileTimeToFileTime(CurFileDate, CurFileDate2);
  SetFileTime(DestF.Handle, @CurFileDate2, nil, @CurFileDate2);
  FreeAndNil(SourceF);
//  if SourceFOpen then
//    CloseFile(SourceF);
end;

/////////////////////////////         old (3.x - 4.0.0) bzip2 stuff    ////////////////////////

const
  BZ_RUN              = 0;
  BZ_FLUSH            = 1;
  BZ_FINISH           = 2;

  BZ_OK               = 0;
  BZ_RUN_OK           = 1;
  BZ_FLUSH_OK         = 2;
  BZ_FINISH_OK        = 3;
  BZ_STREAM_END       = 4;
  BZ_SEQUENCE_ERROR   = (-1);
  BZ_PARAM_ERROR      = (-2);
  BZ_MEM_ERROR        = (-3);
  BZ_DATA_ERROR       = (-4);
  BZ_DATA_ERROR_MAGIC = (-5);
  BZ_IO_ERROR         = (-6);
  BZ_UNEXPECTED_EOF   = (-7);
  BZ_OUTBUFF_FULL     = (-8);
  BZ_CONFIG_ERROR     = (-9);

  SBzlibDataError = 'bzlib: Compressed data is corrupted';
  SBzlibInternalError = 'bzlib: Internal error. Code %d';


function BZAllocMem(AppData: Pointer; Items, Size: Cardinal): Pointer; stdcall;
begin
  try
    GetMem(Result, Items * Size);
  except
    { trap any exception, because zlib expects a NULL result if it's out
      of memory }
    Result := nil;
  end;
end;

procedure BZFreeMem(AppData, Block: Pointer); stdcall;
begin
  FreeMem(Block);
end;

function MemCheck(const Code: Integer): Integer;
begin
  if Code = BZ_MEM_ERROR then
    OutOfMemoryError;
  Result := Code;
end;

procedure InitStream(var strm: TBZStreamRec);
begin
  FillChar(strm, SizeOf(strm), 0);
  with strm do begin
    zalloc := BZAllocMem;
    zfree := BZFreeMem;
  end;
end;

const
  BufferSize = 65536;

function BZCustomInflateData(const ReadProc: TZReadProc; const WriteProc: TZWriteProc;
  const ExtraData: Longint; var Adler: Longint): Boolean;
var
  strm: TBZStreamRec;
  I, O: Pointer;
  Finished: Boolean;
  A: Longint;
  Res: Integer;
begin
  Result := False;
  I := nil;
  O := nil;
  try
    GetMem(I, BufferSize);
    GetMem(O, BufferSize);
    InitStream(strm);
    A := 1;
    Res := MemCheck(BZ2_bzDecompressInit(strm, 0, 0));
    if Res <> BZ_OK then
      raise ECompressInternalError.CreateFmt(SBzlibInternalError, [Res]);
    try
      Finished := False;
      repeat
        if strm.avail_in = 0 then begin
          strm.next_in := I;
          strm.total_in := 0;
          if not Finished then begin
            strm.avail_in := ReadProc(I^, BufferSize, ExtraData);
            if strm.avail_in = 0 then
              Finished := True;
          end;
        end;
        strm.next_out := O;
        strm.avail_out := BufferSize;
        strm.total_out := 0;
        Res := MemCheck(BZ2_bzDecompress(strm));
        case Res of
          BZ_OK, BZ_STREAM_END: ;
          BZ_DATA_ERROR, BZ_DATA_ERROR_MAGIC: raise ECompressDataError.Create(SBzlibDataError);
        else
          raise ECompressInternalError.CreateFmt(SBzlibInternalError, [Res]);
        end;
        if strm.total_out <> 0 then begin
          if Integer(WriteProc(O^, strm.total_out, ExtraData)) <> strm.total_out then
            Exit;
          A := CalcAdler32(A, O^, strm.total_out);
        end;
      until Res = BZ_STREAM_END;
      Adler := A;
    finally
      Res := MemCheck(BZ2_bzDecompressEnd(strm));
      if Res <> BZ_OK then
        raise ECompressInternalError.CreateFmt(SBzlibInternalError, [Res]);
    end;
  finally
    if Assigned(O) then FreeMem(O);
    if Assigned(I) then FreeMem(I);
  end;
  Result := True;
end;

end.