unit RebuildScript;

interface

function GetInstallScript:string;
function GetCompressMethodName:string;
function GetLanguageFile(i:integer):string;
procedure AddEmbeddedFiles;

implementation
uses Windows, SysUtils, Struct, Main, MyTypes, PathFunc, MD5, SHA1;

var
  Res:TBigString;
  CurSlice:integer;

procedure write(s:string);
begin
  Res.AppendString(s);
end;

procedure writeln(s:string);
begin
  Res.AppendString(s+#13#10);
end;

function VerToStr (Ver: Cardinal; ServicePack: Word): String;
var
  Digits: Integer;
begin
  with TSetupVersionDataVersion(Ver) do begin
    Digits := 2;
    if Minor mod 10 = 0 then begin
      Dec (Digits);
      Minor := Minor div 10;
    end;
    FmtStr (Result, '%d.%.*d', [Major, Digits, Minor]);
    if Build <> 0 then
      Result := Result + Format('.%d', [Build]);
    if ServicePack <> 0 then begin
      Result := Result + ' Service Pack ' + IntToStr(Hi(ServicePack));
      if Lo(ServicePack) <> 0 then
        Result := Result + Format('.%d', [Lo(ServicePack)]);
    end;
  end;
end;

function RegRootToStr(const re:TSetupRegistryEntry):string;
begin
  with re do begin
    if rootkey=HKEY_CLASSES_ROOT then Result:='HKCR'
    else if rootkey=HKEY_CURRENT_USER then Result:='HKCU'
    else if rootkey=HKEY_LOCAL_MACHINE then Result:='HKLM'
    else if rootkey=HKEY_USERS then Result:='HKU'
    else if rootkey=HKEY_CURRENT_CONFIG then Result:='HKCC'
    else Result:='';
    if Result<>'' then
      if ro32bit in Options then Result:=Result+'32'
      else if ro64bit in Options then Result:=Result+'64';
  end;
end;

function DoubleQuotes(s:string):string;
var i:integer;
begin
  for i:=length(s) downto 1 do
    if s[i]='"' then Insert('"',s,i);
  Result:=s;
end;

function UnUnicode(s:string):string;
var
  i,n:integer;
begin
  i:=1; Result:='';
  while i<=length(s) do begin
    if Ver>=4202 then n:=(byte(s[i+1]) shl 8) or byte(s[i])
    else n:=byte(s[i]);//(byte(s[i]) shl 8) or byte(s[i+1]);
    if (n>=32) and (n<=127) then Result:=Result+char(n)
    else Result:=Result+'<'+{IntToStr(n)} Format('%4.4x', [n])+'>';
    if Ver>=4202 then inc(i,2) else inc(i);
  end;
end;

procedure StrParam(DisplayName, Value:string; UseQuotes: boolean = true);
begin
  if Value<>'' then
  begin
    if UseQuotes then
      write(DisplayName + ': "' + DoubleQuotes(Value) + '"; ')
    else
      write(DisplayName + ': ' + Value + '; ');  
  end;
end;

procedure IntParam(DisplayName:string; Value:Int64);
begin
  if Value<>0 then write(DisplayName+': "'+IntToStr(Value)+'"; ');
end;

procedure StrConst(DisplayName, Value:string; ShowAlways:boolean=false);
begin
  if ShowAlways or (Value<>'') then writeln(DisplayName+'='+Value);
end;

procedure IntConst(DisplayName:string; Value:Int64);
begin
  writeln(DisplayName+'='+IntToStr(Value));
end;

function SaltToStr(Salt: TSetupSalt) : string;
var
  i: integer;
begin
  Result := '';
  for i := Low(Salt) to High(Salt) do
  begin
    Result := Result + IntToHex(Salt[i], 2);
  end;
  Result := LowerCase(Result);
end;

procedure PrintVersions(MinVersion, OnlyBelowVersion: TMySetupVersionData);
begin
  with MinVersion do
    if (WinVersion>$04000000) or (NTVersion>$04000000) then
      write('MinVersion: '+VerToStr(WinVersion,0)+','+VerToStr(NTVersion,NTServicePack)+'; ');
  with OnlyBelowVersion do
    if (WinVersion<>0) or (NTVersion<>0) then
      write('OnlyBelowVersion: '+VerToStr(WinVersion,0)+','+VerToStr(NTVersion,NTServicePack)+'; ');
end;

procedure PrintRegistryEntry(const re:TSetupRegistryEntry);
  function OptStr(Opt: TSetupRegistryOption):string;
  begin
    case Opt of
      roCreateValueIfDoesntExist         : Result:='createvalueifdoesntexist';
      roUninsDeleteValue                 : Result:='uninsdeletevalue';
      roUninsClearValue                  : Result:='uninsclearvalue';
      roUninsDeleteEntireKey             : Result:='uninsdeletekey';
      roUninsDeleteEntireKeyIfEmpty      : Result:='uninsdeletekeyifempty';
      roPreserveStringType               : Result:='preservestringtype';
      roDeleteKey                        : Result:='deletekey';
      roDeleteValue                      : Result:='deletevalue';
      roNoError                          : Result:='noerror';
      roDontCreateKey                    : Result:='dontcreatekey';
    else Result:='';
    end;
  end;
var
  s,t:string;
  i:dword;
  o:TSetupRegistryOption;
begin
  with re do begin
    write('Root: '+RegRootToStr(re)+'; ');
    StrParam('Subkey',Subkey);
    StrParam('ValueName',ValueName);
    t:='';
    if Typ<>rtNone then begin
      write('ValueType: ');
      case Typ of
        rtString: begin write('String; '); t:=ValueData; end;
        rtExpandString: begin write('ExpandSZ; '); t:=ValueData; end;
        rtDword: begin write('Dword; ');
          if Ver>=4104 then t:=ValueData else begin
            Move(ValueData[1],i,4); t:=Format('$%x', [i]);
          end;
        end;
        rtQWord: begin write('Qword; '); t := ValueData; end;
        rtBinary: begin write('Binary; ');
          t:='';
          for i:=1 to length(ValueData) do t:=t+IntToHex(byte(ValueData[i]),2)+' ';
          if length(t)>0 then SetLength(t,length(t)-1);
        end;
        rtMultiString: begin write('MultiSZ; '); t:=ValueData;
          repeat
            i:=pos(#0,t);
            if i=0 then break;
            t[i]:='{'; insert('break}',s,i+1);
          until false;
        end;
      end;
    end;
    StrParam('ValueData',t);
    StrParam('Components',Components, false);
    StrParam('Tasks',Tasks, false);
    StrParam('Languages',Languages);
    StrParam('Check',Check);
    StrParam('BeforeInstall',BeforeInstall);
    StrParam('AfterInstall',AfterInstall);

    PrintVersions(MinVersion, OnlyBelowVersion);

    s:='';
    for o:=Low(o) to High(o) do if o in Options then begin
      t:=OptStr(o);
      if t<>'' then s:=s+t+' ';
    end;
    if s<>'' then write('Flags: '+s);
    writeln('');
  end;
end;

procedure PrintRunEntry(const re:TSetupRunEntry);
  function OptStr(Opt: TSetupRunOption):string;
  begin
    case Opt of
      roShellExec           : Result:='shellexec';
      roSkipIfDoesntExist   : Result:='skipifdoesntexist';
      roPostInstall         : Result:='postinstall';
      roUnchecked           : Result:='unchecked';
      roSkipIfSilent        : Result:='skipifsilent';
      roSkipIfNotSilent     : Result:='skipifnotsilent';
      roHideWizard          : Result:='hidewizard';
      roRun32Bit            : Result:='32bit';
      roRun64Bit            : Result:='64bit';
      roRunAsOriginalUser   : Result:='';
    else Result:='';
    end;
  end;
var
  s, t: string;
  o:TSetupRunOption;
begin
  with re do begin
    StrParam('Filename',Name);
    StrParam('Parameters',Parameters);
    StrParam('WorkingDir',WorkingDir);
    StrParam('RunOnceId',RunOnceId);
    StrParam('StatusMsg',StatusMsg);
    StrParam('Description',Description);
    StrParam('Components',Components, false);
    StrParam('Tasks',Tasks, false);
    StrParam('Languages',Languages);
    StrParam('Check',Check);
    StrParam('BeforeInstall',BeforeInstall);
    StrParam('AfterInstall',AfterInstall);

    PrintVersions(MinVersion, OnlyBelowVersion);

    for o:=Low(o) to High(o) do if o in Options then begin
      t:=OptStr(o);
      if t<>'' then s:=s+t+' ';
    end;
    case Wait of
      rwNoWait: s := s + 'nowait';
      rwWaitUntilIdle: s := s + 'waituntilidle';
    end;
    if s<>'' then write('Flags: '+s);

    writeln('');
  end;
end;

procedure PrintTaskEntry(const te:TSetupTaskEntry);
begin
  with te do begin
    StrParam('Name',Name);
    StrParam('Description',Description);
    StrParam('GroupDescription',GroupDescription);
    StrParam('Components',Components);
    StrParam('Languages',Languages);
    StrParam('Check',Check);

    PrintVersions(MinVersion, OnlyBelowVersion);

    writeln('');
  end;
end;

procedure PrintComponentEntry(const ce:TSetupComponentEntry);
var
  s:string;
  i:integer;
begin
  with ce do begin
    StrParam('Name',Name);
    StrParam('Description',Description);
    s:=Types;
    for i:=1 to length(s) do if s[i]=',' then s[i]:=' ';
    StrParam('Types',s);
    IntParam('ExtraDiskSpaceRequired',Int64(ExtraDiskSpaceRequired));
    StrParam('Languages',Languages);
    StrParam('Check',Check);

    PrintVersions(MinVersion, OnlyBelowVersion);

    writeln('');
  end;
end;

procedure PrintTypeEntry(const te:TSetupTypeEntry);
begin
  with te do begin
    StrParam('Name',Name);
    StrParam('Description',Description);
    StrParam('Languages',Languages);
    StrParam('Check',Check);

    PrintVersions(MinVersion, OnlyBelowVersion);

    writeln('');
  end;
end;

procedure PrintFileEntry(const fe:TSetupFileEntry);
  function OptStr(Opt: TSetupFileOption):string;
  begin
    case Opt of
      foConfirmOverwrite         : Result:='confirmoverwrite';
      foUninsNeverUninstall      : Result:='uninsneveruninstall';
      foRestartReplace           : Result:='restartreplace';
      foDeleteAfterInstall       : Result:='deleteafterinstall';
      foRegisterServer           : Result:='regserver';
      foRegisterTypeLib          : Result:='regtypelib';
      foSharedFile               : Result:='sharedfile';
      foCompareTimeStamp         : Result:='comparetimestamp';
      foFontIsntTrueType         : Result:='fontisnttruetype';
      foSkipIfSourceDoesntExist  : Result:='skipifsourcedoesntexist';
      foOverwriteReadOnly        : Result:='overwritereadonly';
      foOverwriteSameVersion     : Result:='';
      foCustomDestName           : Result:='';
      foOnlyIfDestFileExists     : Result:='onlyifdestfileexists';
      foNoRegError               : Result:='noregerror';
      foUninsRestartDelete       : Result:='uninsrestartdelete';
      foOnlyIfDoesntExist        : Result:='onlyifdoesntexist';
      foIgnoreVersion            : Result:='ignoreversion';
      foPromptIfOlder            : Result:='promptifolder';
      foDontCopy                 : Result:='dontcopy';
      foUninsRemoveReadOnly      : Result:='uninsremovereadonly';
      foRecurseSubDirsExternal   : Result:='';
      fo32bit                    : Result:='32bit';
      fo64bit                    : Result:='64bit';
      foExternalSizePreset       : Result:='';
      foSetNTFSCompression       : Result:='setntfscomptression';
      foUnsetNTFSCompression     : Result:='unsetntfscomptression';
      foGacInstall               : Result:='gacinstall';
    else Result:='';
    end;
  end;
var
  s,t:string;
  o:TSetupFileOption;
  Opts:TMySetupFileOptions;
begin
//  if ((fe.FileType in [ftUninstExe, ftRegSvrExe, ftFake]) or not (fe.FileType in [ftUserFile, ftFakeFile]))
//    and not ExtractEmbedded then exit;
  if fe.FileType<>ftUserFile then exit;
  with fe do begin
    if LocationEntry<>-1 then
      with PSetupFileLocationEntry(Entries[seFileLocation][LocationEntry])^ do begin
        if FirstSlice<>LastSlice then
          writeln('; the following file spans '+GetSliceName(FirstSlice)+' to '+GetSliceName(LastSlice))
        else if FirstSlice<>CurSlice then
          writeln('; the following file starts on '+GetSliceName(FirstSlice));
        CurSlice:=LastSlice;
      end;
    if (LocationEntry=-1) then write('; ');
    StrParam('Source',SourceFilename);
    StrParam('DestDir',RemoveBackslashUnlessRoot(DestDir));
    StrParam('DestName',DestName);
    StrParam('FontInstall',InstallFontName);
    StrParam('Components',Components, false);
    StrParam('Tasks',Tasks, false);
    StrParam('Languages',Languages);
    StrParam('Check',Check);
    StrParam('BeforeInstall',BeforeInstall);
    StrParam('AfterInstall',AfterInstall);

    PrintVersions(MinVersion, OnlyBelowVersion);

    s:='';
    Opts:=Options;
    if foDeleteAfterInstall in Opts then Exclude(Opts, foUninsNeverUninstall);
    for o:=Low(o) to High(o) do if o in Opts then begin
      t:=OptStr(o);
      if t<>'' then s:=s+t+' ';
    end;
    if s<>'' then write('Flags: '+s);
    writeln('');
  end;
end;

procedure PrintDeleteEntry(const de: TSetupDeleteEntry);
  function TypeStr(Typ: TSetupDeleteType):string;
  begin
    case Typ of
      dfFiles             : Result:='files';
      dfFilesAndOrSubdirs : Result:='filesandordirs';
      dfDirIfEmpty        : Result:='dirifempty';
    end;  
  end;
begin
  with de do begin
    StrParam('Type', TypeStr(DeleteType), false);
    StrParam('Name',Name);
    StrParam('Components',Components, false);
    StrParam('Tasks',Tasks, false);
    StrParam('Languages',Languages);
    StrParam('Check',Check, false);
    StrParam('BeforeInstall',BeforeInstall);
    StrParam('AfterInstall',AfterInstall);
    writeln('');
  end;
end;

procedure PrintCustomMessageEntry(const ce:TSetupCustomMessageEntry);
var
  s,v:string;
begin
  with ce do begin
    s := Name;
    if (LangIndex>=0) and (ExtractEmbedded or (Entries[seLanguage].Count>1)) then
      s:=PSetupLanguageEntry(Entries[seLanguage][LangIndex])^.Name+'.'+s;
    v := StringReplace(Value, #13#10, '%n', [rfReplaceAll]);
    StrConst(s, v);
  end;
end;

procedure PrintLangOptions(const le:TSetupLanguageEntry);
begin
  with le do begin
    StrConst('LanguageName', UnUnicode(LanguageName));
    StrConst('LanguageID', '$'+IntToHex(LanguageID,4));
    IntConst('LanguageCodePage', LanguageCodePage);
    StrConst('DialogFontName', DialogFontName);
    StrConst('TitleFontName', TitleFontName);
    StrConst('WelcomeFontName', WelcomeFontName);
    StrConst('CopyrightFontName', CopyrightFontName);
    IntConst('DialogFontSize', DialogFontSize);
    IntConst('TitleFontSize', TitleFontSize);
    IntConst('WelcomeFontSize', WelcomeFontSize);
    IntConst('CopyrightFontSize', CopyrightFontSize);
    if (RightToLeft) then StrConst('RightToLeft', 'yes');
  end;
end;

procedure PrintIconEntry(const ie:TSetupIconEntry);
var
  s:string;
begin
  with ie do begin
    StrParam('Name',IconName);
    StrParam('Filename',Filename);
    StrParam('Parameters',Parameters);
    StrParam('WorkingDir',WorkingDir);
    StrParam('IconFilename',IconFilename);
    if IconIndex<>0 then write('IconIndex: '+IntToStr(IconIndex)+'; ');
    StrParam('Comment',Comment);
    StrParam('Components',Components, false);
    StrParam('Tasks',Tasks, false);
    StrParam('Languages',Languages);
    StrParam('Check',Check);
    StrParam('BeforeInstall',BeforeInstall);
    StrParam('AfterInstall',AfterInstall);

    PrintVersions(MinVersion, OnlyBelowVersion);

    s:='';
    case CloseOnExit of
      icYes: s:=s+'closeonexit ';
      icNo: s:=s+'dontcloseonexit ';
    end;
    case ShowCmd of
      SW_SHOWMAXIMIZED: s:=s+'runmaximized ';
      SW_SHOWMINNOACTIVE: s:=s+'runminimized ';
    end;
    if s<>'' then write('Flags: '+s);
    writeln('');
  end;
end;

procedure PrintDirEntry(const de: TSetupDirEntry);
  function OptStr(Opt: TSetupDirOption):string;
  begin
    case Opt of
      doUninsNeverUninstall      : Result:='uninsneveruninstall';
      doDeleteAfterInstall       : Result:='deleteafterinstall';
      doUninsAlwaysUninstall     : Result:='uninsalwaysuninstall';
      doSetNTFSCompression       : Result:='setntfscomptression';
      doUnsetNTFSCompression     : Result:='unsetntfscomptression';
    else Result:='';
    end;
  end;
var
  s,t:string;
  Opts: TMySetupDirOptions;
  o: TSetupDirOption;
begin
  StrParam('Name', de.DirName);
  Opts := de.Options;
  for o:=Low(o) to High(o) do if o in Opts then begin
    t := OptStr(o);
    if t<>'' then s := s + t + ' ';
  end;
  if s<>'' then write('Flags: '+s);
  writeln('');
end;

procedure PrintIniEntry(const ie: TSetupIniEntry);
  function OptStr(Opt: TSetupIniOption):string;
  begin
    case Opt of
      ioCreateKeyIfDoesntExist      : Result:='createkeyifdoesntexist';
      ioUninsDeleteEntry            : Result:='uninsdeleteentry';
      ioUninsDeleteEntireSection    : Result:='uninsdeletesection';
      ioUninsDeleteSectionIfEmpty   : Result:='uninsdeletesectionifempty';
    else Result:='';
    end;
  end;
var
  s,t:string;
  Opts: TMySetupIniOptions;
  o: TSetupIniOption;  
begin
  StrParam('FileName', ie.Filename);
  StrParam('Section', ie.Section);
  StrParam('Entry', ie.Entry);
  StrParam('Value', ie.Value);

  Opts := ie.Options;
  for o:=Low(o) to High(o) do if o in Opts then begin
    t := OptStr(o);
    if t<>'' then s := s + t + ' ';
  end;
  if s<>'' then write('Flags: '+s);
  writeln('');
end;

function GetCompressMethodName:string;
begin
  case SetupHeader.CompressMethod of
    cmStored: Result:='none';
    cmZip: Result:='zip';
    cmBzip: Result:='bzip2';
    cmLZMA: Result:='lzma';
    cmLZMA2: Result:='lzma2';
  end;
end;

function GetInnoVersionStr() : string;
begin
  Result := FileVersion;
  if (VerIsUnicode) then
    Result := Result + ' (Unicode)';
end;

function GetBaseFileName() : string;
var
  name: string;
begin
  name := SetupHeader.BaseFilename;
  if (name = '') then
  begin
    name := ExtractFileName(SetupLdrOriginalFilename);
	// Remove .exe extenstion if have one
	if (Pos('.exe', name) = Length(name) - 3) then
	  name := Copy(name, 1, Length(name) - 4);
  end;
  Result := name;
end;

function MaybeToRtf(Name, Contents:string):string;
begin
  if copy(Contents, 1, 5)='{\rtf' then Name:=ChangeFileExt(Name, '.rtf');
  Result:=Name;
end;

procedure AddFakeRtfOrTxtFile(Name, Contents:string);
begin
  AddFakeFile(MaybeToRtf(Name, Contents), Contents);
end;

function GetLanguageFile(i:integer):string;
begin
  Res.Init;
  writeln('[LangOptions]');
  PrintLangOptions(PSetupLanguageEntry(Entries[seLanguage][i])^);
  Result:=Res.CopyAsString;
  Res.Destroy;
end;

procedure PrintLanguageEntry(const le:TSetupLanguageEntry);
begin
  with le do begin                                      // these filenames must be mirrored in AddEmbeddedFiles()
    StrParam('Name', Name);
    StrParam('MessagesFile', 'embedded\'+Name+'.isl');
    if LicenseText<>'' then StrParam('LicenseFile', MaybeToRtf('embedded\'+Name+'License.txt', LicenseText));
    if InfoBeforeText<>'' then StrParam('InfoBeforeFile', MaybeToRtf('embedded\'+Name+'InfoBefore.txt', InfoBeforeText));
    if InfoAfterText<>'' then StrParam('InfoAfterFile', MaybeToRtf('embedded\'+Name+'InfoAfter.txt', InfoAfterText));
  end;
  writeln('');
end;

procedure PrintSetupHeader;
  function ArchStr(a: TSetupProcessorArchitecture):string;
  begin
    case a of
      paX86         : Result:='x86';
      paAMD64       : Result:='x64';
      paIA64        : Result:='ia64';
    else Result:='';
    end;
  end;
  function Hash2Str(var SetupHeader: TMySetupHeader) : string;
  begin
    if (Ver < 5309) then
      Result := MD5DigestToString(SetupHeader.PasswordHash_MD5)
    else
      Result := SHA1DigestToString(SetupHeader.PasswordHash_SHA1);
  end;
  function Priv2Str(Priv: TMySetupPrivileges) : string;
  begin
    case Priv of
      prNone        : Result:='none';
      prPowerUser   : Result:='poweruser';
      prLowest      : Result:='lowest';
    else Result:='';
    end;
  end;
  function DisPage2Str(a: TMySetupDisablePage):string;
  begin
    case a of
      dpAuto   : Result:='auto';
      dpNo     : Result:='no';
      dpYes    : Result:='yes';
    else Result:='';
    end;
  end;
var
  pa: TSetupProcessorArchitecture;
  s,t: string;
begin
  writeln(';InnoSetupVersion=' + GetInnoVersionStr());
  writeln('');

  writeln('[Setup]');
  StrConst('AppName', SetupHeader.AppName, True);
  StrConst('AppVerName', SetupHeader.AppVerName);
  StrConst('AppVersion', SetupHeader.AppVersion);
  StrConst('AppPublisher', SetupHeader.AppPublisher);
  StrConst('AppPublisherURL', SetupHeader.AppPublisherURL);
  StrConst('AppSupportURL', SetupHeader.AppSupportURL);
  StrConst('AppUpdatesURL', SetupHeader.AppUpdatesURL);
  StrConst('AppMutex', SetupHeader.AppMutex);
  if (shCreateAppDir in SetupHeader.Options) then
    writeln('DefaultDirName='+SetupHeader.DefaultDirName)
  else
    writeln('CreateAppDir=no');
  if (SetupHeader.DefaultGroupName <> '(Default)') then
    StrConst('DefaultGroupName', SetupHeader.DefaultGroupName);
  StrConst('UninstallDisplayIcon', SetupHeader.UninstallDisplayIcon);
  StrConst('UninstallDisplayName', SetupHeader.UninstallDisplayName);
  if (SetupHeader.UninstallFilesDir <> '{app}') then
    StrConst('UninstallFilesDir', SetupHeader.UninstallFilesDir);
  writeln('OutputBaseFilename=' + GetBaseFilename());
  writeln('Compression='+GetCompressMethodName);
  if SetupHeader.EncryptionUsed then writeln('; Encryption=yes');
  if (Ver > 4202) and (SetupHeader.EncryptionUsed or (shPassword in SetupHeader.Options)) then
  begin
    writeln('; PasswordHash=' + Hash2Str(SetupHeader));
    writeln('; PasswordSalt=' + SaltToStr(SetupHeader.PasswordSalt));
  end;  
  s:='';
  for pa:=Low(pa) to High(pa) do if pa in SetupHeader.ArchitecturesAllowed then begin
    t:=ArchStr(pa);
    if t<>'' then s:=s+t+' ';
  end;
  if s<>'' then writeln('ArchitecturesAllowed='+s);
  s:='';
  for pa:=Low(pa) to High(pa) do if pa in SetupHeader.ArchitecturesInstallIn64BitMode then begin
    t:=ArchStr(pa);
    if t<>'' then s:=s+t+' ';
  end;
  if s<>'' then writeln('ArchitecturesInstallIn64BitMode='+s);
  if (Ver<5310) and not(shUninstallable in SetupHeader.Options) then
    writeln('Uninstallable=no')
  else if not SameText(SetupHeader.Uninstallable, 'yes') then
    StrConst('Uninstallable', SetupHeader.Uninstallable);
  if (SetupHeader.PrivilegesRequired <> prAdmin) then
    writeln('PrivilegesRequired='+Priv2Str(SetupHeader.PrivilegesRequired));
  if (Int64(SetupHeader.ExtraDiskSpaceRequired) > 0) then
    IntConst('ExtraDiskSpaceRequired',Int64(SetupHeader.ExtraDiskSpaceRequired));
  if (SetupHeader.DisableDirPage <> dpNo) then
    StrConst('DisableDirPage', DisPage2Str(SetupHeader.DisableDirPage));
  if (SetupHeader.DisableProgramGroupPage <> dpNo) then
    StrConst('DisableProgramGroupPage', DisPage2Str(SetupHeader.DisableProgramGroupPage));
  if (shChangesAssociations in SetupHeader.Options) then
    StrConst('ChangesAssociations', 'yes');
  if (shAllowNoIcons in SetupHeader.Options) then
    StrConst('AllowNoIcons', 'yes');
  // mirror these filenames in AddEmbeddedFiles()
  if ExtractEmbedded then begin
    with SetupHeader do begin
      if LicenseText<>'' then writeln('LicenseFile='+MaybeToRtf('embedded\License.txt', LicenseText));
      if InfoBeforeText<>'' then writeln('InfoBeforeFile='+MaybeToRtf('embedded\InfoBefore.txt', InfoBeforeText));
      if InfoAfterText<>'' then writeln('InfoAfterFile='+MaybeToRtf('embedded\InfoAfter.txt', InfoAfterText));
    end;
    if WizardImage<>'' then writeln('WizardImageFile=embedded\WizardImage.bmp');
    if WizardSmallImage<>'' then writeln('WizardSmallImageFile=embedded\WizardSmallImage.bmp');
    if TimeStampsInUTC then writeln(';TimeStampsInUTC=yes');
  end;
end;

procedure AddEmbeddedFiles;
var i:integer;
begin
  with SetupHeader do begin      // mirror these filenames in procedures above
    if LicenseText<>'' then AddFakeRtfOrTxtFile('embedded\License.txt', LicenseText);
    if InfoBeforeText<>'' then AddFakeRtfOrTxtFile('embedded\InfoBefore.txt', InfoBeforeText);
    if InfoAfterText<>'' then AddFakeRtfOrTxtFile('embedded\InfoAfter.txt', InfoAfterText);
    if CompiledCodeText<>'' then AddFakeFile('embedded\CompiledCode.bin',CompiledCodeText);
  end;
  if WizardImage<>'' then AddFakeFile('embedded\WizardImage.bmp', WizardImage);
  if WizardSmallImage<>'' then AddFakeFile('embedded\WizardSmallImage.bmp', WizardSmallImage);
  if DecompDll<>'' then AddFakeFile('embedded\decompressor.dll', DecompDll);

  for i:=0 to SetupHeader.NumLanguageEntries-1 do
    with PSetupLanguageEntry(Entries[SeLanguage][i])^ do begin
      AddFakeFile('embedded\'+Name+'.isl', GetLanguageFile(i));
      if LicenseText<>'' then AddFakeRtfOrTxtFile('embedded\'+Name+'License.txt', LicenseText);
      if InfoBeforeText<>'' then AddFakeRtfOrTxtFile('embedded\'+Name+'InfoBefore.txt', InfoBeforeText);
      if InfoAfterText<>'' then AddFakeRtfOrTxtFile('embedded\'+Name+'InfoAfter.txt', InfoAfterText);
    end;
end;

function GetInstallScript:string;
var
  i:integer;
begin
  Res.Init;

  // Add UTF-8 BOM to script start for Unicode versions.
  if (VerIsUnicode) then write(#$EF#$BB#$BF);

  PrintSetupHeader;

  CurSlice:=0;
  if Entries[seFile].Count>0 then begin
    writeln(#13#10'[Files]');
    for i:=0 to Entries[seFile].Count-1 do
      PrintFileEntry(PSetupFileEntry(Entries[seFile][i])^);
  end;
  if Entries[seDir].Count>0 then begin
    writeln(#13#10'[Dirs]');
    for i:=0 to Entries[seDir].Count-1 do
      PrintDirEntry(PSetupDirEntry(Entries[seDir][i])^);
  end;
  if Entries[seRegistry].Count>0 then begin
    writeln(#13#10'[Registry]');
    for i:=0 to Entries[seRegistry].Count-1 do
      PrintRegistryEntry(PSetupRegistryEntry(Entries[seRegistry][i])^);
  end;
  if Entries[seIni].Count>0 then begin
    writeln(#13#10'[INI]');
    for i:=0 to Entries[seIni].Count-1 do
      PrintIniEntry(PSetupIniEntry(Entries[seIni][i])^);
  end;
  if Entries[seRun].Count>0 then begin
    writeln(#13#10'[Run]');
    for i:=0 to Entries[seRun].Count-1 do
      PrintRunEntry(PSetupRunEntry(Entries[seRun][i])^);
  end;
  if Entries[seUninstallRun].Count>0 then begin
    writeln(#13#10'[UninstallRun]');
    for i:=0 to Entries[seUninstallRun].Count-1 do
      PrintRunEntry(PSetupRunEntry(Entries[seUninstallRun][i])^);
  end;
  if Entries[seIcon].Count>0 then begin
    writeln(#13#10'[Icons]');
    for i:=0 to Entries[seIcon].Count-1 do
      PrintIconEntry(PSetupIconEntry(Entries[seIcon][i])^);
  end;
  if Entries[seTask].Count>0 then begin
    writeln(#13#10'[Tasks]');
    for i:=0 to Entries[seTask].Count-1 do
      PrintTaskEntry(PSetupTaskEntry(Entries[seTask][i])^);
  end;
  if Entries[seComponent].Count>0 then begin
    writeln(#13#10'[Components]');
    for i:=0 to Entries[seComponent].Count-1 do
      PrintComponentEntry(PSetupComponentEntry(Entries[seComponent][i])^);
  end;
  if Entries[seInstallDelete].Count>0 then begin
    writeln(#13#10'[InstallDelete]');
    for i:=0 to Entries[seInstallDelete].Count-1 do
      PrintDeleteEntry(PSetupDeleteEntry(Entries[seInstallDelete][i])^);
  end;
  if Entries[seUninstallDelete].Count>0 then begin
    writeln(#13#10'[UninstallDelete]');
    for i:=0 to Entries[seUninstallDelete].Count-1 do
      PrintDeleteEntry(PSetupDeleteEntry(Entries[seUninstallDelete][i])^);
  end;
  if Entries[seType].Count>0 then begin
    writeln(#13#10'[Types]');
    for i:=0 to Entries[seType].Count-1 do
      PrintTypeEntry(PSetupTypeEntry(Entries[seType][i])^);
  end;
  if Entries[seCustomMessage].Count>0 then begin
    writeln(#13#10'[CustomMessages]');
    for i:=0 to Entries[seCustomMessage].Count-1 do
      PrintCustomMessageEntry(PSetupCustomMessageEntry(Entries[seCustomMessage][i])^);
  end;
  if ExtractEmbedded or (Entries[seLanguage].Count>1) then begin
    writeln(#13#10'[Languages]');
    writeln('; These files are stubs');
    writeln('; To achieve better results after recompilation, use the real language files');
    for i:=0 to Entries[seLanguage].Count-1 do
      PrintLanguageEntry(PSetupLanguageEntry(Entries[seLanguage][i])^);
  end;
  Result:=Res.CopyAsString;
  Res.Destroy;
end;


end.